/* bbhutil.h */
/* Copyright (c) 1995 by Robert Marsa */

#ifndef _BBHUTIL_H
#define _BBHUTIL_H

#include <bbh_types.h>
#include <sys/types.h>

#define MAXRANK 5

#ifdef __cplusplus
extern "C" {
#endif

BBH_C *upcase(BBH_C *s);

#ifndef HAVE_STRDUP
BBH_C *strdup(const BBH_C *s);
#endif

BBH_I strcmp_nc(const BBH_C *s1, const BBH_C *s2);

BBH_V rdvcpy(BBH_D *r,BBH_D *s, BBH_I len);

/* return l2 norm of vec */
BBH_D l2norm(BBH_I N, BBH_D *vec);

/* allocate a vector of size BBH_Ds, print message if
   not enough memory */
BBH_D * vec_alloc(BBH_I size);

/* returns a vector of size BBH_Is;  
   prints error message and exits if not enough memory */
BBH_I * ivec_alloc(BBH_I size);

BBH_V get_int_param(const BBH_C *p_file, BBH_C *name, BBH_I def,
                   BBH_I *p, BBH_I size);

BBH_V get_real_param(const BBH_C *p_file, BBH_C *name, BBH_I def,
                    BBH_D *p, BBH_I size);

BBH_V get_str_param(const BBH_C *p_file, BBH_C *name, BBH_I def,
                   BBH_C **p, BBH_I size);

/* search ascii file for a line of the form
   name := value.
   Assign value to p.
*/
BBH_V get_param(const BBH_C *p_file, BBH_C *name, BBH_I def, 
               BBH_C *type, BBH_I size, BBH_V *p);

BBH_V get_param_nc(const BBH_C *p_file, BBH_C *name, BBH_I def, 
                   BBH_C *type, BBH_I size, BBH_V *p);

typedef struct {
  BBH_C *fn;
  BBH_C *ds;
	BBH_I vers;
  BBH_I mode;
  BBH_I fid;
  BBH_I state;
  BBH_I sdsnum;
  BBH_I nslabs;
  BBH_I rank;
  BBH_D *tmv;
	time_t mtime;
} hdf_data;

BBH_I gft_init(BBH_V);
BBH_V gft_del(BBH_V);
BBH_V gft_clear(const BBH_I ind);
BBH_C * gft_make_file_name(const BBH_C *s);
BBH_I gft_search_list(const BBH_C *fn, hdf_data *list, BBH_I n);
BBH_V gft_dump_hdf_data(hdf_data *df);
BBH_I gft_newstruct(hdf_data **df, BBH_I len);
BBH_V gft_close_all(BBH_V);
BBH_I gft_set_d_type(BBH_I type);
BBH_I gft_check_file(BBH_I ind);
BBH_I gft_set_file_rnpl(BBH_I ind);
BBH_I gft_create(const BBH_C *nm);
BBH_I gft_open_rw(const BBH_C *nm);
BBH_I gft_open_rd(const BBH_C *nm);
BBH_I gft_close(const BBH_C *nm);

BBH_I GFT_extract(const BBH_C *fname, /* in: file name */
          const BBH_I tlev, /* in: time level to retrieve (-ntlev - -1 : 1 - ntlev) */
          BBH_C **func_name, /* name of this grid function */
          BBH_I *ntlev, /* number of time levels available in file */
          BBH_D **tvec, /* vector of time values */
          BBH_D *time, /* time of this slice */
          BBH_I *drank, /* rank of data set */
          BBH_I **shape, /* shape of data set */
          BBH_C ***cnames, /* names of coordinates */
          BBH_D ***coords, /* values of coordinates */
          BBH_D **data ); /* actual data */

/* takes constant args and returns array */
BBH_D *gft_make_bbox(BBH_I rank, ...);

/*  returns full function name:
      func_name(c0,c1,c2,...,t);
    rank is the spacial rank
    cnames[rank] contains the spacial coordinate names
*/
BBH_C *gft_make_full_func_name(BBH_C *func_name, BBH_I rank, BBH_C **cnames);

BBH_I gft_write(const BBH_C *func_name, /* name of this grid function */
                BBH_D time, /* time of this slice */
                BBH_I *shape, /* shape of data set */
                BBH_I rank, /* rank of data set */
                BBH_D *data ); /* actual data */

BBH_I gft_write_brief(const BBH_C *func_name, /* name of this grid function */
                      BBH_D time, /* time of this slice */
                      BBH_I *shape, /* shape of data set */
                      BBH_I rank, /* rank of data set */
                      BBH_D *data ); /* actual data */

BBH_I gft_write_bbox(const BBH_C *func_name, /* name of this grid function */
                    BBH_D time, /* time of this slice */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *coords, /* values of coordinate bboxes */
                    BBH_D *data ); /* actual data */

BBH_I gft_write_full(const BBH_C *func_name, /* name of this grid function */
                    BBH_D time, /* time of this slice */
                    BBH_I *shape, /* shape of data set */
                    BBH_C **cnames, /* names of coordinates */
                    BBH_I rank, /* rank of data set */
                    BBH_D *coords, /* values of coordinates */
                    BBH_D *data ); /* actual data */

BBH_I gft_read_shape(const BBH_C *file_name, /* IN: */
                      const BBH_I level, /* IN: */
                      BBH_I *shape); /* OUT: */

BBH_I gft_read_full(const BBH_C *file_name, /* IN: name of this file */
                    BBH_I level, /* IN: time level */
                    BBH_I *shape, /* OUT: shape of data set */
                    BBH_C **cnames, /* OUT: names of coordinates */
                    BBH_I rank, /* IN: rank of data set */
                    BBH_D *time, /* OUT: time value */
                    BBH_D *coords, /* OUT: values of coordinates */
                    BBH_D *data ); /* OUT: actual data */

BBH_I gft_read_brief(const BBH_C *file_name, /* IN: name of this file */
                    BBH_I level, /* IN: time level */
                    BBH_D *data ); /* OUT: actual data */

BBH_I gft_read_id_str_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_C **param, BBH_I nparam);
BBH_I gft_read_id_int_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_I *param, BBH_I nparam);
BBH_I gft_read_id_float_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_D *param, BBH_I nparam);
BBH_I gft_read_2idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *datanm1,
                    BBH_D *datan ); /* actual data */
BBH_I gft_read_1idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *datan ); /* actual data */
BBH_I gft_read_idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *data); /* actual data */

BBH_I gft_write_id_str_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_C **param, BBH_I nparam);
BBH_I gft_write_id_int_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_I *param, BBH_I nparam);
BBH_I gft_write_id_float_p(const BBH_C *file_name, const BBH_C *param_name,
                        BBH_D *param, BBH_I nparam);
BBH_I gft_write_2idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *datanm1,
                    BBH_D *datan ); /* actual data */
BBH_I gft_write_1idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *datan ); /* actual data */
BBH_I gft_write_idata(const BBH_C *file_name, /* name of idata file */
                    const BBH_C *func_name, /* name of this grid function */
                    BBH_I *shape, /* shape of data set */
                    BBH_I rank, /* rank of data set */
                    BBH_D *data); /* actual data */

#ifdef __cplusplus
}
#endif

#endif
